# stdin.tcl
# (c) 1996 Sun Microsystems Laboratories (Stephen Uhler)

# See the file "license.terms" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.

# The following is an event-driven command loop for testing with tclsh
# with command history:
#	!<pattern> run 1st command matching pattern
#	!<pattern>/xxx/yyy/ as above, but substitute yyy for xxx

package provide stdin 1.1

set stdinMaxHistory 100
append stdinHistory {}
proc StdinRead {prompt} {
    global stdinCommand stdinWait stdinHistory stdinMaxHistory
    if [eof stdin] {
	set stdinWait 1
	fileevent stdin readable {}
	return
    }
    append stdinCommand [gets stdin]\n
    if {[info complete $stdinCommand]} {
	set s1 ""; set s2 ""
	if {[regexp {^!([^/]+)(/([^/]+)/([^/]*))?} $stdinCommand \
		    {} history {} s1 s2]} {
	    set match [lsearch -regexp $stdinHistory ^$history]
	    if {$match >= 0} {
	    	set stdinCommand [lindex $stdinHistory $match]
	    	puts stderr $stdinCommand
	    	if {$s1 != ""} {
		    regsub -- $s1 $stdinCommand $s2 stdinCommand
		    puts stderr $stdinCommand
	    	}
	    } else {
	    	puts -nonewline "No \"$history\" in history list\n$prompt"
		flush stdout
		set stdinCommand ""
		return
	    }
	}
	if {$stdinCommand != ""} {
	    set stdinHistory [lrange "[list $stdinCommand] $stdinHistory" \
		     0 $stdinMaxHistory]
	}
        catch {uplevel #0  $stdinCommand} result
        puts -nonewline "$result\n$prompt"
        flush stdout
        set stdinCommand ""
    }
}

proc Stdin_Start {{prompt "% "}} {
    global stdinWait
    set stdinCommand ""
    puts -nonewline $prompt
    flush stdout
    fileevent stdin readable [list StdinRead $prompt]
    vwait stdinWait
}

proc history {{count 100}} {
    global stdinHistory
    puts stderr [join $stdinHistory \n]
}	

if {[info procs bgerror] == ""} {
    proc bgerror {args} {
	global errorInfo
	puts stderr $errorInfo
    }
}					
