# Tcl Wiki Server - $Id: edit.tcl,v 1.3 2000/01/05 13:55:40 mbl Exp $
# 
# Copyright (c) 1997-99 Michel Beaudouin-Lafon, mbl@daimi.au.dk
# See the file "licence" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.

# Wiki server command
#	must be loaded in the Wiki namespace

# send a form to the client to create a new page
#
proc SendNewPageForm {siteName pageName} {
	GetSite $siteName theSite
	
	# create empty page description
	# if the 'anchor' field exists, it is the title for the new page
	InitPage $siteName $pageName {} thePage
	set thePage(source) ""
	if [catch {cgi_import anchor}] {
		set thePage(title) [Name2Title $pageName]
	} else {
		# capitalize anchor name
		set anchor "[string toupper [string range $anchor 0 0]][string range $anchor 1 end]"
		set thePage(title) $anchor
	}
	
	# get user name
	variable Wiki
	set wikiuser ""
	if ![catch {cgi_import_cookie wikiuser}] {
		set Wiki(user) $wikiuser
	}
	Debug "SendNewPageForm:wikiuser=<$wikiuser>"

	# get the page type
	if [catch {cgi_import type}] {
		set type "wiki"
	} else {
		set thePage(type) $type
	}
	
	# create form
	HttpHeader new theSite thePage
	${siteName}::PageTitle new thePage
	
	cgi_body {
		# generate the form. The on Submit handler is used for preview mode
		# the code for the doSubmit must be generated by PageFooter
		cgi_form [CurrentURL $siteName $pageName] onSubmit=doSubmit(this) [$type-EditFormAttr thePage] {
			# generate header
			${siteName}::PageHeader new thePage
			cgi_export title=$thePage(title)
			cgi_export baseversion=0
			cgi_export type=$type
			
			# type-dependant body
			$type-EditForm thePage
			
			# generate footer
			# cmd-save will be called if the user submits the form
			# cmd-preview will be called it the user clicks "Preview"
			cgi_export cmd=save
			${siteName}::PageFooter new thePage
		}
	}
	unset thePage(source)
	catch {unset Wiki(user)}
}

# send a form to the client to edit a page
#	if the version being edited is not the current one,
#	it will nevertheless be saved as the current one:
#	editing an older version means that we want to replace the current version by the older one.
#
proc SendEditPageForm {pageVar} {
	upvar $pageVar thePage
	set siteName $thePage(site) 
	GetSite $siteName theSite
	
	# load source
	LoadPage thePage source
	
	# if it's not the current version, we need the current version
	# to set the correct baseversion
	if {$thePage(current)} {
		set version $thePage(version)
	} else {
		GetPage $thePage(site) $thePage(name) curPage
		set version $curPage(version)
	}
	
	# get user name
	variable Wiki
	set wikiuser ""
	if ![catch {cgi_import_cookie wikiuser}] {
		set Wiki(user) $wikiuser
	}
	Debug "SendEditPageForm:wikiuser=<$wikiuser>"

	# get page type
	if [catch {set type $thePage(type)}] {
		set type "wiki"
	}
	
	# create form to edit the page
	HttpHeader edit theSite thePage
	${siteName}::PageTitle edit thePage

	cgi_body {
		# generate the form. The on Submit handler is used for preview mode
		# the code for the doSubmit must be generated by PageFooter
		cgi_form [CurrentURL $thePage(site) $thePage(name)] onSubmit=doSubmit(this) [$type-EditFormAttr thePage] {
			# generate header
			${siteName}::PageHeader edit thePage
			cgi_export title=$thePage(title)
			cgi_export baseversion=$version
			cgi_export type=$type

			# type-dependant body
			if {[lsearch -exact $Wiki(types) $type] >= 0} {
				$type-EditForm thePage
			}
			
			# generate footer
			# cmd-save will be called if the user submits the form
			cgi_export cmd=save
			
			${siteName}::PageFooter edit thePage
		}
	}
	
	# unset source to save space
	unset thePage(source)
	catch {unset Wiki(user)}
}

# edit the current version of a page or create a new page if it does not exist.
# if a version is specified, the form will contain the old version of the page but will
# be saved into the current version if the user submits the page.
# this is typically used to replace the current version with an old one, e.g.
# if something has been accidentally deleted.
#
proc cmd-edit {site page} {
	if [catch {cgi_import version}] {
		set version {}
	}
	
	if {$version == {}} {
		# edit current version or create new page if the page does not exist
		if [PageExists $site $page] {
			# check access rights: edit existing page requires edit right
			if {! [CheckAccess $site e]} {
				return
			}
			
			GetPage $site $page thePage
			SendEditPageForm thePage
		} else {
			# check access rights: create new page requires create right
			if {! [CheckAccess $site c]} {
				return
			}
			
			SendNewPageForm $site $page
		}
	} else {
		# check access rights: edit from old version requires edit _and_ history right
		if {! [CheckAccess $site e*h]} {
			return
		}
			
		# edit old version, or error if it does not exist
		if [PageExists $site $page $version] {
			GetOldPage $site $page $version thePage
			SendEditPageForm thePage
			unset thePage
		} else {
			error "Version $version of page $page does no exist"
		}
	}
}

# -- Log --
#
# mbl 05/01/00	changed 'v' right to 'h' right in cmd-edit
#
# mbl 29/06/99	turned puts into Debug
#
