# Tcl Wiki Server - $Id: get.tcl,v 1.3 1999/06/29 23:58:28 mbl Exp $
# 
# Copyright (c) 1997-99 Michel Beaudouin-Lafon, mbl@daimi.au.dk
# See the file "licence" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.

# Wiki server command
#	must be loaded in the Wiki namespace


# translate a page into html if necessary and send it to the client
#
proc SendPage {pageVar} {
	upvar $pageVar thePage
	GetSite $thePage(site) theSite

	# check whether to translate the current version or a previous one
	if {$thePage(current)} {
		# check if the html file exists and is more recent, if so just return it
		# ### we also need to check whether the config file has changed more
		# ### recently than the html file and has been loaded since then
		set translate 1
		set cache ""
		if {$theSite(cacheHtml)} {
			Debug "SendPage: Checking for cached HTML"
			set cache "cache"
			set htmlFile [HtmlFile $thePage(site) $thePage(name)]
			if [file exists $htmlFile] {
				set htmlDate [file mtime $htmlFile]
				set wikiFile [CurrentFile $thePage(site) $thePage(name)]
#puts "File exists, wiki mtime= [file mtime $wikiFile], html mtime= $htmlDate"
				if {[file mtime $wikiFile] <= $htmlDate
				 && [info exists theSite(configDate)] && $theSite(configDate) < $htmlDate} {
					set translate 0
					Debug "SendPage $thePage(name): using cached HTML"
				}
			}
		}
		
		# translate current version to html and return relocation to server
		if {$translate} {
			Debug "SendPage $thePage(name): translating to HTML"
			Page2Html thePage $cache
		} else {
			# send relocation to server
			cgi_location [HtmlURL $thePage(site) $thePage(name)]
		}
	} else {
		# translate to html (no cache here)
		Page2Html thePage
	}
}

# get the current version of a file, or an old version, or
# create a new page if the page does not exist
#
proc cmd-get {site page} {
	if [catch {cgi_import version}] {
		set version {}
	}
	
	if {$version == {}} {
		# current version: get page or create it if it does not exist
		if [PageExists $site $page] {
			# check access rights: get current version requires read right
			if {! [CheckAccess $site r]} {
				return
			}
			
			GetPage $site $page thePage
			SendPage thePage
		} else {
			# check access rights: create page requires creation right
			if {! [CheckAccess $site c]} {
				return
			}
			
			SendNewPageForm $site $page
		}
	} else {
		# check access rights: access to [previous versions requires history 
		if {! [CheckAccess $site h]} {
			return
		}
		
		# retrieve previous version. error if it does not exist
		if [PageExists $site $page $version] {
			GetOldPage $site $page $version thePage
			SendPage thePage
			unset thePage
		} else {
			error "Version $version of page $page does no exist"
		}
	}
}

# -- Log --
#
# mbl 29/06/99	changed test on dates from < to <= since the resolution is 1 second and it is
#		very common that the html file is generated in the same second as the wiki file is saved.
#
# mbl 29/06/99	changed puts into Debug
#
