# Tcl Wiki Server - $Id: history.tcl,v 1.1 1999/06/28 23:18:48 mbl Exp $
# 
# Copyright (c) 1997-99 Michel Beaudouin-Lafon, mbl@daimi.au.dk
# See the file "licence" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.

# Wiki server command
#	must be loaded in the Wiki namespace

# create a page with the available versions of a page
#
proc cmd-history {site page} {
	GetPage $site $page thePage
	GetSite $site theSite
	GetFormat $site theFormat
	set siteName $site
	
	# check access rights: require history access
	if {! [CheckAccess $site h]} {
		return
	}
	
	HttpHeader history theSite thePage
	${siteName}::PageTitle history thePage
	
	# ### should we limit history of versions ?
#	set old [expr [clock seconds] - 60*60*24*7]
	
	cgi_body {
		# header
		${siteName}::PageHeader history thePage
		
		# get date of current version
		lset {lastday time} [DayTime [clock scan $thePage(date)]]

		# start list with current version
		#	header
		regsub {(.*)} [DateTrans theSite $lastday] $theFormat(dateHeader) line
		cgi_puts $line
		#	start list
		lset {open item close} $theFormat(pageList)
		cgi_puts $open
		#	current page item
		cgi_puts "$item$time: [OldPageRef thePage]"
		
		# previous versions
		for {set version [expr $thePage(version) -1]} {$version > 0} {incr version -1} {
			if {! [PageExists $site $page $version]} {
				continue
			}
			GetOldPage $site $page $version oldPage
			
			set when [clock scan $oldPage(date)]
#			if {$when < $old} {
#				# too old: forget about this version and the previous ones
#				break
#			}
			
			lset {day time} [DayTime $when]
			if {$day != $lastday} {
				# close previous list
				cgi_puts $close
				
				# header with new date
				regsub {(.*)} [DateTrans theSite $day] $theFormat(dateHeader) line
				cgi_puts $line
				
				# start new list
				cgi_puts $open
				set lastday $day
			}
			
			# add page item
			cgi_puts "$item$time: [OldPageRef oldPage]"
			
			unset oldPage
		}
		
		cgi_puts $close

		# form to generate differences between two versions
		${siteName}::PageDiffForm history thePage
		
		# footer
		${siteName}::PageFooter history thePage
	}
}
