# Tcl Wiki Server - $Id: preview.tcl,v 1.2 1999/06/29 23:58:35 mbl Exp $
# 
# Copyright (c) 1997-99 Michel Beaudouin-Lafon, mbl@daimi.au.dk
# See the file "licence" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.

# Wiki server command
#	must be loaded in the Wiki namespace

# send a preview of an edited page
# this is called by cmd-save and extracts the necessary input from the submitted form
# with cgi_import.
#
# (### this is very similar to cmd-save - we might want to share code)
#
proc cmd-preview {site page} {
	GetPage $site $page thePage
	GetSite $site theSite
	variable Wiki
	
	# check access rights: require edit right to preview
	if {! [CheckAccess $site e]} {
		return
	}
	
	# curent version: the version saved in $thePage(file)
	# base version: the version that was used by the user to create the new version
	# new version: the version edited by the user, not yet saved

	# save current version
#puts "Previewing version $thePage(version) of $thePage(name)"
	set curversion $thePage(version)
 	
	# import values from form submission
	cgi_import title
	if [catch {cgi_import newtitle}] {
		set newtitle $title
	}
	cgi_import description
	cgi_import category
	cgi_import newcategory
	cgi_import baseversion
	if [catch {cgi_import source}] {
		set source ""
	}
	if [catch {cgi_import accents}] {
		set accents 0
	}
	if [catch {cgi_import author}] {
		set author ""
	}
		
	if [catch {cgi_import type}] {
		set type "wiki"
	}

	# convert character pairs to accentuated characters
	if {$accents} {
		set newtitle [Accents $newtitle]
		set newcategory [Accents $newcategory]
		set source [Accents $source]
		set author [Accents $author]
	}
	
	# newcategory is not empty if the user typed in a new category
	if {$newcategory != ""} {
		set category $newcategory
	}
	
	# compute new version number
	set newversion [expr $curversion + 1]
	
	# create preview version of page
	set file [file join $theSite(root) $page.preview]
	array set previewPage [array get thePage]
	array set previewPage [list \
		title $newtitle \
		description $description \
		category $category \
		version $newversion \
		file $file
	]
	if {$type != "wiki"} {
		set previewPage(type) $type
	}
	
	# set author and creator if they are specified
	if {$author == ""} {
		catch {unset previewPage(author)}
	} else {
		# creator is autor of version #1
		if {$newversion == 1} {
			set previewPage(creator) $author
		}
		set previewPage(author) $author
	}
	
	# type-dependant processing
	if {$type != "wiki"} {
		if [catch "$type-PreviewPage previewPage" src] {
			# could not save the page
			# ### need better handling
			error "Could not preview page $previewPage(name): $src"
		} else {
			set source $src
		}
	}
	
	# update version date
	set now [Date]
	set previewPage(date) $now
	if {$newversion == 1} {
		set previewPage(datecreated) $now
	}

#puts "  saving preview version $newversion of $thePage(name)"
	# save new version in page file:
	# open file
	if [catch {open $file w} fd] {
		error "error creating preview page: $fd"
	}
	
	# update version date
	set now [Date]
	set previewPage(date) $now
	if {$newversion == 1} {
		set previewPage(datecreated) $now
	}

	# write headers
	foreach key $Wiki(keys) {
		if [info exist previewPage($key)] {
			puts $fd "$key: $previewPage($key)"
		}
	}
	# write delimiter empty line
	puts $fd ""
	
	# write content and close file
	puts -nonewline $fd $source
	close $fd
	
	# send page
	Page2Html previewPage preview

	# discard preview page
	unset previewPage
	catch {file delete $file}
#puts "done"
}

# -- Log --
#
# mbl 29/06/99	commented out puts
#
