# Tcl Wiki Server - $Id: recent.tcl,v 1.1 1999/06/28 23:19:11 mbl Exp $
# 
# Copyright (c) 1997-99 Michel Beaudouin-Lafon, mbl@daimi.au.dk
# See the file "licence" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.

# Wiki server command
#	must be loaded in the Wiki namespace

# show list of recently modified pages
#	(page argument is ignored)
#
proc cmd-recent {site page} {
	variable Wiki
	GetSite $site theSite
	GetFormat $site theFormat
	set siteName $site

	# check access rights: require history access to get recent changes
	if {! [CheckAccess $site h]} {
		return
	}
	
	if {$theSite(cacheHtml)} {
		# see if we have a cached HTML version that we can use
		set htmlfile [HtmlFile $site _recent]
		if {[file exist $htmlfile]} {
			# send relocation to server
			cgi_location [HtmlURL $site _recent]
			return
		}
		
		# open html file for saving output
		if [catch {open $htmlfile w} fdout] {
			error "error opening html page: $fdout"
		}
		cgi_savetohtml $fdout $htmlfile
	}
	
	# output header and title
	HttpHeader recent theSite
	${siteName}::SpecialTitle theSite _recent
	
	# determine the day where we stop: we want at least one week of history,
	# and at least 100 entries (or as much as there are in the history if less than 100)
	# ### should have a settable expiration period and size settable in Config
	set old [expr [clock seconds] - 60*60*24*7]
	set maxentries 100
#puts "1 week ago = $old ([clock format $old])"

	# get 100th entry, or last one if fewer than 100
	if {[llength $theSite(recent)] > $maxentries} {
		lset $Wiki(histKeys) [lindex $theSite(recent) $maxentries]
	} else {
		lset $Wiki(histKeys) [lindex $theSite(recent) end]
	}
	# compute the end of the day of that page
	# date may not exist, e.g. if the 'recent' list is empty
	if [info exist date] {
		set when [clock scan "[Day [clock scan $date]] 00:00:00"]
#puts "100 entries ago = $date, scanning to $when ([clock format $when])" 
		# if that's older than the period we want, extend the period to that date
		if {$when < $old} {
#puts "limiting history to $when"
			set old $when
		}
	}
	
	# go through the history and generate the list
	set recent {}
	set lastday ""
	cgi_body {
		# header
		${siteName}::SpecialHeader theSite _recent [DateTrans theSite [Day $old]]
		
		lset {open item close} $theFormat(pageList)
		foreach change $theSite(recent) {
#puts $change
			lset $Wiki(histKeys) $change
#puts "$page v$version $date"
			set when [clock scan $date]
			if {$when < $old} {
#puts "too old - stopping here"
				# too old: forget about this item and the rest of the list
				break
			}
			
			# create a fake page ('thePage') that can be passed to OldPageRef
			# this is to avoid a call to GetOldPage since we have all the information we need.
			# we only call GetPage to find out whether this is the current version of a page.
			GetPage $site $name aPage
			set current 1
			if {$aPage(version) != $version} {
				set current 0
			}
			foreach key [concat $Wiki(histKeys) site current] {
				set thePage($key) [set $key]
			}

			# output markup to break the list into days
			lset {day time} [DayTime $when]
			if {$day != $lastday} {
				# close previous list if any
				if {$lastday != ""} {
					cgi_puts $close
				}
				
				# header line
				regsub {(.*)} [DateTrans theSite $day] $theFormat(dateHeader) line
				cgi_puts $line
				
				# open list
				cgi_puts $open
				set lastday $day
			}
			
			# put list item
			cgi_puts "$item$time: [OldPageRef thePage]"
			unset thePage
			
			# next page
			lappend recent $change
		}

		# close list
		if {$lastday == ""} {
			p [Dict theSite noChange [DateTrans theSite [Day $old]]]
		} else {
			cgi_puts $close
		}

		# footer
		${siteName}::SpecialFooter theSite _recent
	}

	# close html file
	if {$theSite(cacheHtml)} {
		cgi_savetohtml {}
		close $fdout
	}
	
	# update the list of recent changes since we may have cut its tail
	set theSite(recent) $recent
}
