# Tcl Wiki Server - $Id: sitemap.tcl,v 1.4 1999/07/07 15:15:20 mbl Exp $
# 
# Copyright (c) 1997-99 Michel Beaudouin-Lafon, mbl@daimi.au.dk
# See the file "licence" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.

# Wiki server command
#	must be loaded in the Wiki namespace

# show list of pages by category
#	(page argument is ignored)
#
# ### note: this command gets called only if the sitemap page type is _not_ defined,
#	or if it is defined but the sitemap wiki page does _not_ exist.
#
proc cmd-sitemap {site page} {
	GetSite $site theSite
	GetFormat $site theFormat
	set siteName $site

	# check access rights: require read right
	if {! [CheckAccess $site r]} {
		return
	}
	
	if {$theSite(cacheHtml)} {
		# see if we have a cached HTML version that we can use
		set htmlfile [HtmlFile $site _sitemap]
		if {[file exist $htmlfile]} {
			# send relocation to server
			cgi_location [HtmlURL $site _sitemap]
			return
		}
		
		# open html file for saving output
		if [catch {open $htmlfile w} fdout] {
			error "error opening html page: $fdout"
		}
		cgi_savetohtml $fdout $htmlfile
	}

	# output header and title
	HttpHeader sitemap theSite
	${siteName}::SpecialTitle theSite _sitemap
	
	cgi_body {
		# header
		${siteName}::SpecialHeader theSite _sitemap
		
		# create list of pages by category
		# ### the effect of this is to load _all_ pages
		# ### maybe we could unload the pages that were not loaded before
		# ### or have a general mechanism to unload unused pages, based on last access date
		foreach p [AllPages theSite] {
			if {$p == "_sitemap"} {
				# we don't want the site map in the site map...
				continue
			}
			GetPage $site $p aPage
			lappend category($aPage(category)) [list $p [PageRef aPage]]
			# check if the page has the same title as the category
			if {[Title2Name $aPage(category)] == $p} {
				set title($aPage(category)) [PageLink aPage]
			}
		}
		
		# create sitemap
		lset {open item close} $theFormat(pageList)
		foreach cat [lsort [array names category]] {
			# header for the category
			# make it a link if there is a page in this category with the same name
			if [info exist title($cat)] {
				set line $title($cat)
			} else {
				set line $cat
			}
			regsub {(.*)} $line $theFormat(categoryHeader) line
			cgi_puts $line
			
			# list of pages
			cgi_puts $open
			foreach pr [lsort $category($cat)] {
				lset {p ref} $pr
				cgi_puts $item$ref
			}
			cgi_puts $close
		}
		
		# footer
		${siteName}::SpecialFooter theSite _sitemap
		
	}
	
	# close html file
	if {$theSite(cacheHtml)} {
		cgi_savetohtml {}
		close $fdout
	}
	
	# update the list of categories since we have it at hand
	set theSite(categories) [lsort [array names category]]
}

# -- Log --
#
# mbl 28/06/99	sorted the list of categories since it is not sorted anymore when
#		displayed in the category menu, to accomodate editing the sitemap.
#
# mbl 30/06/99	added a test to not list the sitemap page in the site map.
#
