# Tcl Wiki Server - $Id: xedit.tcl,v 1.2 2000/01/05 13:55:40 mbl Exp $
# 
# Copyright (c) 1997-99 Michel Beaudouin-Lafon, mbl@daimi.au.dk
# See the file "licence" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.

# Wiki server command
#	must be loaded in the Wiki namespace

# send the page in text/x-wiki format
#
proc SendXWikiEditPageForm {pageVar} {
	upvar $pageVar thePage
	set siteName $thePage(site) 
	GetSite $siteName theSite
	
	# load source
	LoadPage thePage source
	
	# if it's not the current version, we need the current version
	# to set the correct baseversion
	if {$thePage(current)} {
		set version $thePage(version)
	} else {
		GetPage $thePage(site) $thePage(name) curPage
		set version $curPage(version)
	}

	# respond to client
	cgi_http_head {
		# tell client to not cache this page unless it is a reply to a POST.
		# This dirty trick avoids the "Data Missing" error under Navigator
		# when hitting the 'back' button after to get back to the result
		# of a previous edit.
		# Also, be careful that Location is not the first header
		# otherwise it would be interpreted as a re-location to be handled
		# by the server (see cgi_puts in ext.tcl). Aarghh!!!
		if [info exist thePage(postReply)] {
			set url [CurrentURL $theSite(name) $thePage(name)]
			cgi_puts "Content-Location: $url"
			cgi_puts "Location: $url"
		} else {
			cgi_pragma no-cache
		}
		# tell client that we only deal with ISO-Latin-1
		cgi_content_type "text/x-wiki; charset=ISO-8859-1"
	}

	# send page
 	foreach key {title description category version date} {
		cgi_puts "$key: $thePage($key)"
	}
	cgi_puts "newcategory: "
	cgi_puts "url: [CurrentURL $siteName $thePage(name)]"
	cgi_puts ""
	cgi_puts $thePage(source)
	
	# unset source to save space
	unset thePage(source)
}

# edit a page with an external editor
# this is similar to edit but the contents of the page is sent
# as headers + plain text with MIME type text/x-wiki.
# by configuring the browser properly, users can edit a page
# with their favorite editor.
#
proc cmd-xedit {site page} {
	if [catch {cgi_import version}] {
		set version {}
	}
	
	if {$version == {}} {
		# edit current version or create new page if the page does not exist
		if [PageExists $site $page] {
			# check access rights: edit existing page requires edit right
			if {! [CheckAccess $site e]} {
				return
			}
			
			GetPage $site $page thePage
			SendXWikiEditPageForm thePage
		} else {
			# check access rights: create new page requires create right
			if {! [CheckAccess $site c]} {
				return
			}
			
			SendNewPageForm $site $page
		}
	} else {
		# check access rights: edit from old version requires edit _and_ history right
		if {! [CheckAccess $site e*h]} {
			return
		}
			
		# edit old version, or error if it does not exist
		if [PageExists $site $page $version] {

			GetOldPage $site $page $version thePage
			SendXWikiEditPageForm thePage
			unset thePage
		} else {
			error "Version $version of page $page does no exist"
		}
	}
}

# -- Log --
#
# mbl 05/01/00	changed 'v' right to 'h' right in cmd-edit
#
