# Tcl Wiki Server - $Id: format.tcl,v 1.7 2005/11/16 00:54:34 mbl Exp $
# 
# Copyright (c) 1997-99 Michel Beaudouin-Lafon, mbl@daimi.au.dk
# See the file "licence" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.

# Wiki format control
#	this file must be loaded into the Wiki namespace
#

#	exportProcs	procs to be exported to site namespace so they can be used in custom format
#	formatProcs	procs to be defined within site namespace so they can be redefined by custom format
#
array set Wiki {
	exportProcs	{GetSite GetFormat Dict PageLinks SiteLinks PageNavigation PageVersion DiffDescription \
			 EditTitle EditDescription EditCategory EditPageType EditAuthor EditAccents EditButtons }
	formatProcs	{PageName PageInfo PageTitle PageHeader PageFooter PageDiffForm SiteSearchForm PreviewBanner \
			 SpecialName SpecialInfo SpecialTitle SpecialHeader SpecialFooter Format InitFormat}
}

# ---- date formatting ----
#
# note that this is also used in non-html output, e.g. to date .wiki files
#
# format the current date/time or the number of seconds given as argument
#	Date		full date and time
#	Day		short date (no time)
#	Time		time without date
#	DayTime		day and time as a two-elements list

proc Date {{date {}}} {
	if {$date == {}} {
		set date [clock seconds]
	}
	clock format $date -format "%a %d %b %Y %H:%M:%S"
}

proc Day {{date {}}} {
	if {$date == {}} {
		set date [clock seconds]
	}
	clock format $date -format "%a %d %b %Y"
}

proc Time {{date {}}} {
	if {$date == {}} {
		set date [clock seconds]
	}
	clock format $date -format "%H:%M:%S"
}

proc DayTime {{date {}}} {
	if {$date == {}} {
		set date [clock seconds]
	}
	clock format $date -format "\{%a %d %b %Y\} \{%H:%M:%S\}"
}

# ---- misc. formatting ----

# output a menu with all the available categories plus a text field to create a new category
#
proc CategoryMenu {siteVar category} {
	upvar $siteVar theSite
	
	cgi_puts "[Dict theSite Category] "
	cgi_select category {
		set categories $theSite(categories)
		if {[llength $categories] == 0 || [lindex $categories 0] != ""} {
			cgi_option "" selected_if_equal=$category
		}
		foreach cat $categories {
			cgi_option $cat selected_if_equal=$category
		}
	}
	cgi_puts " [Dict theSite newCategory] "
	cgi_text newcategory= size=20
}

# output a menu with all the available file types
#
proc EditPageType {pageVar} {
	upvar $pageVar thePage
	GetSite $thePage(site) theSite
	variable Wiki

	if [catch {set pageType $thePage(type)}] {
		set pageType wiki
	}

	cgi_puts "[Dict theSite changePageType] "
	set url "$thePage(name).wiki?[cgi_set cmd edit]&[cgi_set anchor $thePage(title)]"
	cgi_select newtype onChange=document.location='$url&type='+this.options\[this.selectedIndex\].value {
		foreach type $Wiki(types) {
			cgi_option [Dict theSite ${type}PageType] value=$type selected_if_equal=$pageType
		}
	}
}

# return a string with the version number and date of the page,
# and a button to retrieve the changes since last version
#
proc PageVersion {pageVar} {
	upvar $pageVar thePage
	GetSite $thePage(site) theSite

	set v "[Dict theSite Version $thePage(version)], [DateTrans theSite $thePage(date)] [PageAuthor thePage]"
	if {$thePage(version) != 1 && [info exist thePage(datecreated)]} {
		append v " - [Dict theSite datecreated] [DateTrans theSite $thePage(datecreated)] [PageCreator thePage]"
	}
	return $v
}

# output a text field to edit the title of the page
#
proc EditTitle {pageVar {size 40}} {
	upvar $pageVar thePage
	GetSite $thePage(site) theSite
	
	cgi_puts [Dict theSite enterTitle]
	cgi_text newtitle=$thePage(title) size=$size
}

# output a text field to edit the description of the page
#
proc EditDescription {pageVar {size 80}} {
	upvar $pageVar thePage
	GetSite $thePage(site) theSite
	
	if {$thePage(description) == {}} {
		cgi_puts [Dict theSite enterDescription]
		br
		cgi_text description= size=$size
	} else {
		cgi_text description=$thePage(description) size=$size
	}
}

# output a menu and field to edit the category of a page
#
proc EditCategory {pageVar} {
	upvar $pageVar thePage
	GetSite $thePage(site) theSite
	
	CategoryMenu theSite $thePage(category)
}

# output a field to edit the author name
#
proc EditAuthor {pageVar} {
	upvar $pageVar thePage
	GetSite $thePage(site) theSite
	variable Wiki

	if [catch {set wikiuser $Wiki(user)}] {
		set wikiuser ""
	}
	cgi_puts [Dict theSite author]
	cgi_text author=$wikiuser size=20
}

# output a field to set accents conversion
#
proc EditAccents {pageVar} {
	upvar $pageVar thePage
	GetSite $thePage(site) theSite
	
	cgi_checkbox accents
	cgi_puts [Dict theSite accentsCheckbox]
}

# output the buttons to save/preview/reset/cancel an edited page
#	type is "new" or "edit"
# output the doSubmit javascripts function
#

set PreviewScript {
	preview=0
	function doSubmit(form) {
		if (preview) {
			previewWindow=window.open('','wikipreview','scrollbars=yes')
			form.target = 'wikipreview'
			form.cmd.value = 'preview'
		} else {
			form.target = ''
			form.cmd.value = 'save'
			closePreview()
		}
	}
	function closePreview() {
		if (typeof (previewWindow) == 'object'
		&& typeof (previewWindow.close) == 'function') {
			previewWindow.close()
		}
	}
}

proc EditButtons {type pageVar} {
	upvar $pageVar thePage
	GetSite $thePage(site) theSite
	
	cgi_puts <p>
	
	# save button
	cgi_submit_button =[Dict theSite saveButton] onClick=preview=0
	
	# preview mode: if the user clicks preview, we create a new window
	# and direct the output of the preview command to that window.
	# if the user clicks save, we delete the preview window.
	variable PreviewScript
	cgi_javascript {
		cgi_puts $PreviewScript
		cgi_puts "document.writeln('<input type=submit value=\"[Dict theSite previewButton]\" onClick=\"preview=1\">')"
	}

	# reset button
	cgi_reset_button [Dict theSite resetButton]

	# back button. The back function
	# does not exist before javascript 1.2 on Navigator 4 and IE 4
        cgi_javascript {
		cgi_puts "if (parseInt(navigator.appVersion) >= '4') {document.writeln('<input type=button value=[Dict theSite cancelButton] onClick=\"closePreview();history.back()\">');}"
	}

	# menu to change the page type (only for new pages)
	if {$type == "new"} {
		EditPageType thePage
	}

	cgi_puts </p>
}

# return a signature string for the author and creator of a page
#
proc PageAuthor {pageVar} {
	upvar $pageVar thePage

	if {! [info exists thePage(author)]} {
		return ""
	}
	set author $thePage(author)
	if {$author == ""} {
		return ""
	}
	return [url \[$author\] [Title2Name $author].wiki]
}

proc PageCreator {pageVar} {
	upvar $pageVar thePage

	if {! [info exists thePage(creator)]} {
		return ""
	}
	set creator $thePage(creator)
	if {$creator == ""} {
		return ""
	}
	return [url \[$creator\] [Title2Name $creator].wiki]
}

# return a description text for diff pages
# the reference and current version are passed in the temporary "diff" field of the page
#
proc DiffDescription {pageVar} {
	upvar $pageVar thePage
	GetSite $thePage(site) theSite
	
	if [catch {lset {refversion version current} $thePage(diff)}] {
		return ""
	}
	set from [Dict theSite diffFrom $refversion]
	if {$current} {
		set to [Dict theSite diffCurrent $version]
	} else {
		set to [Dict theSite diffTo $version]
	}
	return "$from $to"
}

# return a link to the current version of a page
#
proc PageLink {pageVar} {
	upvar $pageVar thePage
	
	return [url $thePage(title) $thePage(name).wiki]
}

# return a one-line reference to the current version of a page
#
proc PageRef {pageVar} {
	upvar $pageVar thePage

	return "[url $thePage(title) $thePage(name).wiki] - $thePage(description) [PageCreator thePage]"
}

# return a one-line reference to an arbitrary version of a page
#
proc OldPageRef {pageVar} {
	upvar $pageVar thePage
	GetSite $thePage(site) theSite
	
	set title "$thePage(title)"
	if {$thePage(current)} {
		append title " ([Dict theSite currentVersion])"
		set version ""
	} else {
		append title " ([Dict theSite Version $thePage(version)])"
		set version "?version=$thePage(version)"
	}
	
	return "[url $title $thePage(name).wiki$version] - $thePage(description) [PageAuthor thePage]"
}

# ---- HTTP headers ----

# generate the HTTP header
#
proc HttpHeader {type siteVar {pageVar {}}} {
	upvar $siteVar theSite
	GetFormat $theSite(name) theFormat
	if {$pageVar != ""} {
		upvar $pageVar thePage
	}
	
	# respond to client
	cgi_http_head {
		# tell client to not cache this page unless it is a reply to a POST.
		# This dirty trick avoids the "Data Missing" error under Navigator
		# when hitting the 'back' button after to get back to the result
		# of a previous edit.
		# Also, be careful that Location is not the first header
		# otherwise it would be interpreted as a re-location to be handled
		# by the server (see cgi_puts in ext.tcl). Aarghh!!!
		if [info exist thePage(postReply)] {
			set url [CurrentURL $theSite(name) $thePage(name)]
			cgi_puts "Content-Location: $url"
			cgi_puts "Location: $url"
		} else {
			cgi_pragma no-cache
		}
		# tell client that we only deal with ISO-Latin-1
		cgi_content_type "text/html; charset=ISO-8859-1"
	}
	
	# send header and style
	if {$type == "preview"} {
		# set the base target to _blank so that each time we click
		# a link in a preview window, the link appears in a new window
		eval cgi_base target=_blank
		# bring the preview window to the front when the previewed page is loaded
		eval cgi_body_args onLoad=focus() $theFormat(bodyargs)
	} else {
		eval cgi_body_args $theFormat(bodyargs)
	}
	cgi_put $theFormat(style)
}	

# ---- title/header/footer for standard pages -----
#
# the "type" argument specifies the type of page being generated:
#	get		display current page (or older version) in HTML
#	preview		display a preview of a page being edited
#	diff		display the changes between previous version and current page
#	edit		form to edit the current page (or an older version)
#	new		form to create a new page
#	history		list of available versions of this page
#

# return a list of links for the commands on a page
# defaults to:
#	Edit: edit this page
#	History: show a history of versions for this page
#	Changes: show changes since previous version
#
proc PageLinks {pageVar} {
	upvar $pageVar thePage
	GetSite $thePage(site) theSite
	
	if {$thePage(current)} {
		set v ""
	} else {
		set v "&version=$thePage(version)"
	}
	
	return [list \
		[url [Dict theSite Edit] $thePage(name).wiki?cmd=edit$v] \
		[url [Dict theSite History] $thePage(name).wiki?cmd=history] \
		[url [Dict theSite Changes] $thePage(name).wiki?cmd=diff$v]]
}

# return a list of links for site-wide commands
# defaults to:
#	Home: go to home page
#	SiteMap: display site map
#	Search: search site
#	Recent: display recent changes
#
proc SiteLinks {siteVar} {
	upvar $siteVar theSite
	
	return [list \
		[url [Dict theSite Home] _home.wiki] \
		[url [Dict theSite SiteMap] _sitemap.wiki] \
		[url [Dict theSite Search] _search.wiki] \
		[url [Dict theSite Recent] _recent.wiki]]
}

# generate the HTML navigation panel
#	pageVar is empty if the panel is not related to a wiki page
#	position is 'top' or 'bottom'
# defaults to:
#	pagelinks | sitelinks
# or
#	sitelinks for site-wide pages (sitemap, recent changes, etc.)
# also adds a ruler before or after the links depending on the position
#
proc PageNavigation {siteVar pageVar position} {
	upvar $siteVar theSite
	
	set links {}
	if {$pageVar != ""} {
		upvar $pageVar thePage
		set links [PageLinks thePage]
	}
	set links [concat $links [SiteLinks theSite]]
	if {$position == "top"} {
		p class=navig [join $links " | "]<hr>
	} else {
		p class=navig <hr>[join $links " | "]
	}
}

# generate a banner for preview windows
#	position is 'top' or 'bottom
proc PreviewBanner {siteVar pageVar position} {
	upvar $siteVar theSite
	
	if {$position == "bottom"} {
		hr
	}
	h1 align=center [Dict theSite preview]
	if {$position == "top"} {
		hr
	}
	
}

# generate the HTML headers and title
#
proc PageTitle {type pageVar} {
	upvar $pageVar thePage
	GetSite $thePage(site) theSite
	GetFormat $thePage(site) theFormat
	
	if {$thePage(current)} {
		set v {}
	} else {
		set v " - [Dict theSite Version $thePage(version)]"
	}
	
	switch -exact $type {
		get {
			cgi_title "$thePage(title)$v"
		}
		preview {
			cgi_title "$thePage(title) ([Dict theSite preview])"
		}
		diff {
			cgi_title "$thePage(title)$v ([Dict theSite diff])"
		}
		edit {
			cgi_title "$thePage(title) ([Dict theSite edit]$v)"
		}
		new {
			cgi_title "$thePage(title) ([Dict theSite new])"
		}
		history {
			cgi_title "$thePage(title) ([Dict theSite history])"
		}
	}
}

# Generate the page name and description
#
proc PageName {type pageVar} {
	upvar $pageVar thePage
	
	h1 class=title [cgi_title]
	switch -exact $type {
		get -
		preview {
			p class=description $thePage(description)
		}
		diff {
			p class=description [DiffDescription thePage]
		}
	}
}

# Generate page information to be added at the end of the page
#
proc PageInfo {type pageVar} {
	upvar $pageVar thePage
	
	if {$type == "get"} {
		hr
		p class=version [PageVersion thePage]
	}
}

# Generate the beginning of the body of a page
#
proc PageHeader {type pageVar} {
	upvar $pageVar thePage
	GetSite $thePage(site) theSite

	if {$type == "preview"} {
		PreviewBanner theSite thePage top
		PageName $type thePage
		hr
		return
	}
	
	PageNavigation theSite thePage top
	PageName $type thePage

	switch -exact $type {
		edit -
		new {
# useless now that we get the original title with the 'anchor' field (see cmd-edit.tcl)
#			if {$type == "new"} {
#				EditTitle thePage
#				br
#			}
			EditDescription thePage
			br
			EditCategory thePage
		}
	}
	
	hr
}

# generate the end of the body of a page
#
proc PageFooter {type pageVar} {
	upvar $pageVar thePage
	GetSite $thePage(site) theSite
	
	PageInfo $type thePage
	
	switch -exact $type {
		edit -
		new {
			br
			EditAuthor thePage
			EditAccents thePage
			br
			EditButtons $type thePage
		}
		preview {
			PreviewBanner theSite thePage bottom
			return
		}
	}
	
	PageNavigation theSite thePage bottom
}

# ---- forms ----

# generate a form to request the diffs between two versions of a page
# type can be 'history' or 'diff' depending on the source page (unused here).
# the text fields to enter the reference and current version numbers are
# initialized from the arguments 'version' and 'refversion'.
# version defaults to the page's version
# refversion defaults to version-1
#
proc PageDiffForm {type pageVar {version {}} {refversion {}}} {
	upvar $pageVar thePage
	GetSite $thePage(site) theSite

	cgi_hr
	if {$version == ""} {
		set version $thePage(version)
	}
	if {$refversion == ""} {
		set refversion [expr $version -1]
	}
	cgi_form [::Wiki::CurrentURL $theSite(name) $thePage(name)] {
		cgi_submit_button =[Dict theSite changes]
		cgi_puts [Dict theSite fromVersion]
		cgi_text refversion=$refversion size=4
		cgi_puts [Dict theSite toVersion]
		cgi_text version=$version size=4
		cgi_export cmd=diff
	}
}

# generate a search form to search the site.
# 'string' is the current search string and is used to initialize
# the search string field.
#
proc SiteSearchForm {siteVar string} {
	upvar $siteVar theSite

	cgi_form [::Wiki::CurrentURL $theSite(name) _search] method=get {
		cgi_text string=$string size=30
		cgi_submit_button =[Dict theSite searchButton]
	}
}

# ---- title/header/footer for special pages -----
#
# page can be
#	_recent		recent changes
#	_sitemap	list of existing pages
#	_search		search or search result page

# generate the title and description of the page
#
proc SpecialName {siteVar page name descr} {
	h1 class=title [cgi_title]
	if {$descr != {}} {
		p class=description $descr
	}
}

# generate page information to be added at the bottom of the page
#
proc SpecialInfo {siteVar page} {
    # nothing
}

# generate the HTML headers and title
#	for _search, the additional args holds the string being searched
#
proc SpecialTitle {siteVar page args} {
	upvar $siteVar theSite
	GetSite $theSite(name) theSite
	GetFormat $theSite(name) theFormat
	
	switch $page {
		_recent {
			cgi_title [Dict theSite recentChanges $theSite(name)]
		}
		_sitemap {
			cgi_title [Dict theSite siteMap $theSite(name)]
		}
		_search {
			if {[llength $args] > 0 && [lindex $args 0] != ""} {
				cgi_title [Dict theSite searchResults [lindex $args 0]]
			} else {
				cgi_title [Dict theSite search $theSite(name)]
			}
		}
	}
}

# generate the beginning of the body of the page
#	for _recent, the additional args holds the date where the page stops
#	for _search, the additional args holds the string being searched
#
proc SpecialHeader {siteVar page args} {
	upvar $siteVar theSite
	
	
	set descr {}
	switch $page {
		_recent {
			set descr [Dict theSite changesSince [lindex $args 0]]
			PageNavigation theSite {} top
		}
		_sitemap {
			::Wiki::GetPage $theSite(name) _sitemap sitemapPage
			PageNavigation theSite sitemapPage top
		}
		_search {
			PageNavigation theSite {} top
		}
	}
	
	SpecialName theSite $page [cgi_title] $descr
	
	hr
}

# generate the end of the body of the page
#
proc SpecialFooter {siteVar page} {
	upvar $siteVar theSite
	
	SpecialInfo theSite $page
	PageNavigation theSite {} bottom
}

# ---- load custom format ----

# this proc loads a custom format.
# It is meant to be called from a site configuration file (<site>/_config),
# i.e. in the site's namespace.
#	name is the name of the format (looked up in lib/wiki/formats)
#	params is a list of format parameters/values
#
proc Format {name {params {}}} {
	global Httpd
	variable Format
	
	# create format file name and make sure it exists
	set formatfile [file join $Httpd(extlibrary) wiki formats $name.tcl]
	if {! [file exists $formatfile] } {
		error "Format $name does not exist"
	}
	
	# source the format file
	source $formatfile
	
	# pass parameters to format's init proc
	catch {InitFormat $params}
	
	# record the format name
	set Format(format) $name
}

# this is the default proc to initialize a format
# it just copies the parameters into the site's Format array.
# Format files typically redefine it to parse the parameters
# differently and/or generate other side effects.
#
proc InitFormat {params} {
	variable Format
	
	array set Format $params
}

# ---- default format ----

# be careful about comments in the list below:
# in fact Tcl does not skip them and they count as entries of the array!
# so be sure that the comment text is exactly one word!!
#

array set DefaultFormat {
	# page
	style		""
	bodyargs	""
	
	# paragraphs
	p	"<p>"
	hr	"<hr>"
	
	# headers
	h1	{<h1>\1</h1>}
	h2	{<h2>\1</h2>}
	h3	{<h3>\1</h3>}
	h4	{<h4>\1</h4>}
	
	# text
	italic	{<em>\1</em>}
	bold	{<strong>\1</strong>}
	code	{<code>\1</code>}

	# lists
	bullet	{<ul> <li> </ul>}
	enum	{<ol> <li> </ol>}
	def	{<dl> {<dt>\1<dd>} </dl>}
	quote	{<blockquote> {} </blockquote>}
	
	# preformatted
	pre		{<pre> </pre>}
	conflict	{<pre> </pre>}

	# diffs
	deleted		{{<font color=red><strike><del>} </del></strike></font>}
	inserted	{{<font color=blue><ins>} </ins></font>}
	
	# summary
	categoryHeader	{<h2>\1</h2>}
	dateHeader	{<h2>\1</h2>}
	pageList	{<ul> <li> </ul>}
	
	# highlighting
	highlightColor	yellow
	highlightParOn	{<table border=0 cellspacing=0 cellpadding=0><tr><td bgcolor=%color>}
	highlightParOff	{</td></tr></table>}
	
}


# note: in bullet and enum, \1 is the bullet or number, if you need it
# note2: beware that the \ must be properly quoted:
#	either include the word in {} or protect the \ (\\)
# note3: in some cases, but not all, a string between [] will be evaluated
#	this won't work for the summary formats

# -- Log --
#
# mbl 22/04/01	added highlightColor, highlightParOn and highlightParOff
#		entries in the format array. Used by bib.tcl.
#		fixed bug with Cancel button on Edit forms with IE
#
# mbl 12/01/01	Added search type for special pages and SiteSearchForm.
#
# mbl 21/01/00	fixed bug in Javascript when Preview button label has more than one word
#
# mbl 28/06/99	CategoryMenu is now in the order of the categories list, to reflect
#		the order in the sitemap.
#
# mbl 06/07/99	added test to CategoryMenu to make sure the first category in the category menu
#		is the empty category.
#
