# Tcl Wiki Server - $Id: lang.tcl,v 1.8 2005/11/16 00:54:34 mbl Exp $
# 
# Copyright (c) 1997-99 Michel Beaudouin-Lafon, mbl@daimi.au.dk
# See the file "licence" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.

# Wiki multi-lingual support
#	this must be loaded into the Wiki namespace

# define here the list of available dictionaries
#
set Wiki(languages) {English French Dansk}

# return the message for the given key in the language of the site
# the string %1 in the message is substituted with the first element of 'args'
#
proc Dict {siteVar key args} {
	upvar $siteVar theSite
	upvar #0 ::Wiki::$theSite(language) dict
	
	if [catch {set trans $dict($key)}] {
		# not found: lookup in English dict
		if [catch {set trans $English($key)}] {
			# not found: return raw msg
			foreach a $args {
				append key " $a"
			}
			return $key
		}
	}
	
	if {$args != {}} {
		set arg [lindex $args 0]
		regsub -all "%1" $trans $arg trans
	}
	
	return $trans
}

# translate a date in the language of the site
# actually we only translate the day and month, assuming the general format is OK
#
proc DateTrans {siteVar date} {
	upvar $siteVar theSite
	upvar #0 ::Wiki::$theSite(language) dict
	
	set ldate [split $date " "]
	if {[llength $ldate] < 3} {
		return $date
	}
	set ldate [lreplace $ldate 0 0 $dict([lindex $ldate 0])]
	set ldate [lreplace $ldate 2 2 $dict([lindex $ldate 2])]
	return [join $ldate]
}

# language-specific dictionaries
#
# be careful about comments in the list below:
# in fact Tcl does not skip them and they count as entries of the array!
# so be sure that the comment text is exactly one word!!
#

array set English {
	# keys
	title		"title"
	description	"description"
	category	"category"
	version		"version"
	date		"date"
	datecreated	"created"
	
	# categoryMenu
	Category	"Category:"
	newCategory	"new category:"
	
	# conflicts
	conflict	"conflict"
	endConflict	"end conflict"
	lastVersion	"last version"
	editedVersion	"edited version"
	currentVersion	"current version"
	
	# navigation
	Edit		"Edit"
	History		"History"
	Changes		"Changes"
	Home		"Home page"
	SiteMap		"Site map"
	Search		"Search"
	Recent		"Recent changes"
	Version		"Version %1"
	Help		"Help"
	
	# titles
	new		"new"
	history		"history"
	edit		"edit"
	preview		"preview"
	diff		"changes"
	diffFrom	"Changes since version %1"
	diffTo		"upto version %1"
	diffCurrent	"upto current version (%1)"
	recentChanges	"Recent changes to %1"
	homePage	"Home page for site %1"
	siteMap		"Site map of %1"
	changesSince	"Changes since %1"
	noChange	"No change since %1"
	diffEmpty	"No change in the body of the page."
	search		"Search"
	searchResults	"Search results for \"%1\""
	searchList	"%1 pages found"
	searchFailed	"No page found"
	bibliography	"bibliography"
	
	# forms
	saveButton	"Save"
	previewButton	"Preview"
	resetButton	"Reset"
	cancelButton	"Cancel"
	enterTitle	"Edit title if necessary (e.g., upper-case letters):"
	enterContent	"Enter the contents of the page:"
	enterDescription	"Enter a one-line description of the page and select/create a category:"
	accentsCheckbox	"Convert accents"
	author		"Author"
	uploadLabel	"File to upload"
	downloadLabel	"Download"
	changes		"Changes"
	fromVersion	"since version"
	toVersion	"upto version"
	searchButton	"search"
	bibtexEntries	"<h3>BibTeX entries: </h3>"

	# types
	changePageType	"Set page type to:"
	uploadFileType	"Type of file to upload:"
	wikiPageType	"Wiki"
	imagePageType	"Image"
	textPageType	"Text"
	htmlPageType	"HTML"
	filePageType	"File"
	sitemapPageType	"Site map"
	docPageType	"Source code doc."
	bibPageType	"BibTeX"

	# errors
	diffPageError	"Could not compute differences: maybe a version is unavailable"
	
	# days
	Mon		Mon
	Tue		Tue
	Wed		Wed
	Thu		Thu
	Fri		Fri
	Sat		Sat
	Sun		Sun
	
	# months
	Jan		Jan
	Feb		Feb
	Mar		Mar
	Apr		Apr
	May		May
	Jun		Jun
	Jul		Jul
	Aug		Aug
	Sep		Sep
	Oct		Oct
	Nov		Nov
	Dec		Dec

	# sizes
	bytes		"bytes"
	Kb		"Kb"
	Mb		"Mb"
}

array set French {
	# keys
	title		"titre"
	description	"description"
	category	"catgorie"
	version		"version"
	date		"date"
	datecreated	"cre"
	
	# categoryMenu
	Category	"Catgorie :"
	newCategory	"nouvelle catgorie :"
	
	# conflicts
	conflict	"conflit"
	endConflict	"fin de conflit"
	lastVersion	"derniere version"
	editedVersion	"version editee"
	currentVersion	"version courante"
	
	# navigation
	Edit		"Editer"
	History		"Historique"
	Changes		"Changements"
	Home		"Page d'accueil"
	Search		"Rechercher"
	SiteMap		"Carte du site"
	diff		"changements"
	diffFrom	"Changements depuis la version %1"
	diffTo		"jusqu' la version %1"
	diffCurrent	"jusqu' la version courante (%1)"
	Recent		"Changements rcents"
	Version		"Version %1"
	Help		"Aide"
	
	# titles
	new		"cration"
	history		"historique"
	edit		"dition"
	preview		"Prvisualisation"
	recentChanges	"Changements rcents du site %1"
	homePage	"Page d'accueil du site %1"
	siteMap		"Carte du site %1"
	changesSince	"Changements depuis %1"
	noChange	"Pas de changement depuis %1"
	diffEmpty	"Pas de changement dans le contenu de la page."
	search		"Recherche"
	searchResults	"Rsultats de la recherche de \"%1\""
	searchList	"%1 pages trouves"
	searchFailed	"Aucun rsultat"
	bibliography	"bibliographie"
	
	# forms
	saveButton	"Enregistrer"
	previewButton	"Prvisualiser"
	resetButton	"Recomposer"
	cancelButton	"Annuler"
	enterTitle	"Modifier le titre si ncessaire (accents, majuscules...) :"
	enterContent	"Entrer le contenu de la page :"
	enterDescription	"Entrer une courte description de la page et choisir ou crer une catgorie :"
	accentsCheckbox	"Convertir les accents"
	author		"Auteur"
	uploadLabel	"Fichier"
	downloadLabel	"Tlcharger"
	changes		"Changements"
	fromVersion	"depuis la version"
	toVersion	"jusqu' la version"
	searchButton	"rechercher"
	bibtexEntries	"<h3>Entres BibTeX : </h3>"

	# types
	changePageType	"Changer le type de la page : "
	uploadFileType	"Type du fichier  tlcharger :"
	wikiPageType	"Wiki"
	imagePageType	"Image"
	textPageType	"Texte"
	htmlPageType	"HTML"
	filePageType	"Fichier"
	sitemapPageType	"Carte du site"
	docPageType	"Source code doc."
	bibPageType	"BibTeX"
	
	# errors
	diffPageError	"Erreur en calculant les diffrences, peut-tre due  l'absence d'une version."
	
	# days
	Mon		lundi
	Tue		mardi
	Wed		mercredi
	Thu		jeudi
	Fri		vendredi
	Sat		samedi
	Sun		dimanche
	
	# months
	Jan		janvier
	Feb		fvrier
	Mar		mars
	Apr		avril
	May		mai
	Jun		juin
	Jul		juillet
	Aug		aot
	Sep		septembre
	Oct		octobre
	Nov		novembre
	Dec		dcembre

	# sizes
	bytes		"octets"
	Kb		"Ko"
	Mb		"Mo"
}

array set Dansk {
	# keys
	title		"titel"
	description	"beskrivelse"
	category	"kategori"
	version		"version"
	date		"dato"
	datecreated	"oprettet"
	
	# categoryMenu
	Category	"Kategori:"
	newCategory	"Ny kategori:"
	
	# conflicts
	conflict	"Konflikt"
	endConflict	"Konflikt slut"
	lastVersion	"nyeste version"
	editedVersion	"opdateret version"
	currentVersion	"nuvrende version"
	
	# navigation
	Edit		"Opdater"
	History		"Historie"
	Changes		"ndringer"
	Home		"Top side"
	SiteMap		"Oversigt"
	Search		"S?gning"
	Recent		"Nyeste opdateringer"
	Version		"Version %1"
	Help		"Hjlp"
	
	# titles
	new		"ny"
	history		"historie"
	edit		"opdater"
	preview		"se ndring"
	diff		"forskel"
	diffFrom	"Forskel siden version %1"
	diffTo		"op til version %1"
	diffCurrent	"op til nuvrende version (%1)"
	recentChanges	"Sidste ndringer til %1"
	homePage	"Top side for site %1"
	siteMap		"Oversigt for %1"
	changesSince	"ndringer siden %1"
	noChange	"Ingen ndringer siden %1"
	diffEmpty	"Ingen ndringer i indholdet af denne side."
	search		"S?gning"
	searchResults	"S?gte efter \"%1\""
	searchList	"%1 sider"
	searchFailed	"Nogen sider"
	bibliography	"bibliography"
	
	# forms
	saveButton	"Gem"
	previewButton	"Se ndring"
	resetButton	"Forfra"
	cancelButton	"Fortryd"
	enterTitle	"Titlen kan ndres (f.eks., store/sm bogstaver):"
	enterContent	"Tilfj indholdet af siden:"
	enterDescription	"Tilfj en en-linies beskrivelse af siden, samt en kategori"
	accentsCheckbox	"Overst specialsymboler"
	author		"Forfatter"
	uploadLabel	"Importer fil"
	downloadLabel	"Inkluder fil"
	changes		"ndringer"
	fromVersion	"siden version"
	toVersion	"til og med version"
	searchButton	"s?gning"
	bibtexEntries	"<h3>BibTeX entries: </h3>"

	# types
	changePageType	"Vlg side type:"
	uploadFileType	"Type af fil der skal importeres:"
	wikiPageType	"Wiki"
	imagePageType	"Billede"
	textPageType	"Tekst"
	htmlPageType	"HTML"
	filePageType	"Fil"
	sitemapPageType	"Oversigt"
	docPageType	"BetaDokumentation"
	bibPageType	"BibTeX"

	# errors
	diffPageError	"Kunne ikke finde forskellen: Mske er en af versionerne ikke tilgngelig"
	
	# days
	Mon		man
	Tue		tir
	Wed		ons
	Thu		tor
	Fri		fre
	Sat		lr
	Sun		sn
	
	# months
	Jan		jan
	Feb		feb
	Mar		mar
	Apr		apr
	May		maj
	Jun		jun
	Jul		jul
	Aug		aug
	Sep		sep
	Oct		okt
	Nov		nov
	Dec		dec

	# sizes
	bytes		"bytes"
	Kb		"Kb"
	Mb		"Mb"
}


# -- Log --
#
# mbl 12/01/01	added strings for search command: Search, search, searchResults, searchButton, searchList, searchFailed
#
# mbl 28/06/99	changed Dict so that it does not generate and error if the entry does not exist
#		but instead returns the English version if it exists, otherwise the code.
#
# mbl 29/06/99	fixed bug in Dict.
#
# mbl 30/06/99	added Help entry.
#
# mbl 06/07/99	added homePage entry and Wiki(languages) global variable
#
# mbl 20/01/00	added Dansk dictionary
#
# mbl 24/01/00	moved bibPageType and docPageType entries from types/bib.tcl and types/doc.tcl
#
# mbl 10/03/00	added units for file sizes
#
# mbl 21/04/01	added "bibliography" entry = title of default bib page
#		added "bibtexEntries" entry = label of textarea in bib pages edit form
#
