# Tcl Wiki Server - $Id: types.tcl,v 1.6 2000/08/14 09:48:00 mbl Exp $
# 
# Copyright (c) 1997-99 Michel Beaudouin-Lafon, mbl@daimi.au.dk
# See the file "licence" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.

# Wiki page types handlers
#	this file must be loaded into the Wiki namespace
#

# support for the list of page types that we can deal with.
# each type (except wiki) is defined in a file in the <types> subdir.
#
# for each type we need the following procs:
#	<type>-EditFormAttr	return attributs to be appended to edit <form> tag
#	<type>-EditForm		generate html to edit the page contents
#	<type>-SavePage		process save page
#	<type>-PreviewPage	process preview page
#	<type>-Page2Html	turn page into HTML
# 

proc ForgetType {type} {
	if {$type == "wiki"} {
		# wiki type is predefined
		return
	}
	
	rename $type-EditFormAttr {}
	rename $type-EditForm {}
	rename $type-SavePage {}
	rename $type-PreviewPage {}
	rename $type-Page2Html {}
}

# --------
# Utility procs for uploading files 
# --------

# upload the file specified in the content field and save it into dir
#
proc UploadContentFile {pageVar dir} {
	upvar $pageVar thePage
	GetSite $thePage(site) theSite
	Debug "UploadFile $thePage(name) in $dir"

	if [catch {cgi_import content}] {
# puts " could not get file"
		error "unspecified file to upload"
	}
	
	set local [cgi_import_filename -local content]
	set remote [cgi_import_filename -remote content]
	if [string length $remote] {
		# create upload dir if necessary
		set uploadDir [file join $theSite(root) $dir]
		if {! [file exist $uploadDir]} {
# puts " creating $uploadDir"
			file mkdir $uploadDir
		}
# we use the name of the page, _not_ the name of the uploaded file
# ### maybe this causes problems?
# ### maybe we need a field with the file name, to specify it if different from uploaded file name
#		set uploadedFile [file join $uploadDir $remote]
		set uploadedFile [file join $uploadDir $thePage(name)]
		# remove existing file if any
		catch {file delete $uploadedFile}
		# move from tmp to upload dir
		file rename $local $uploadedFile
		# set filename in page record
# ### see above
#		set thePage(content) [file join $dir $remote]
#		set thePage(content) [file join $dir $thePage(name)]
# we need a path that will work in a URL, i.e. with '/', irrespective of the platform:
		set thePage(content) $dir/$thePage(name)
# parray thePage
	} else {
# puts " no file specified: file not changed"
		catch {file delete $local}
		if {! [info exist thePage(content)]} {
			set thePage(content) ""
		}
	}
}

# upload the file specified in the content field and return its contents
#
proc UploadContentText {pageVar} {
	upvar $pageVar thePage
	GetSite $thePage(site) theSite
	Debug "UploadContentText $thePage(name)"

	if [catch {cgi_import content}] {
# puts " could not get file"
		error "unspecified file to upload"
	}
	set local [cgi_import_filename -local content]
	set remote [cgi_import_filename -remote content]
	if [string length $remote] {
		if [catch {open $local r} fd] {
			catch {file delete $local}
			error "uploading page $thePage(name): $fd"
		}
		set src [read $fd]
		close $fd
		# remove uploaded file
		catch {file delete $local}
# parray thePage
		return $src
	} else {
# puts " no file specified: contents unchanged"
		# remove local file if any (unlikely)
		catch {file delete $local}
		# return current content of the page if it exists
		# since the current page is already put in the history,
		# we need to retrieve it from there
		if {$thePage(version) == 1} {
			set content ""
		} else {
			GetOldPage $theSite(name) $thePage(name) [expr $thePage(version) -1] oldPage
			# in case the history page is not there
			catch {GetPageContent oldPage} content
		}
		return $content
	}
}

# -- Log --
#
# mbl 28/06/99	split into types/*.tcl to support new extension loading scheme (see wiki.tcl)
#
# mbl 29/06/99	moved wiki type procs in cmds/wiki.tcl to simplify management of wiki type.
#
# mbl 29/06/99	changed puts into Debug or commented it out.
#
# mbl 10/03/00	handled cases where the file to upload is not specified when the page is created.
#
