# Tcl Wiki Server - $Id: bib.tcl,v 1.4 2005/11/16 00:54:34 mbl Exp $
# 
# Copyright (c) 1997-99 Michel Beaudouin-Lafon, mbl@daimi.au.dk
# See the file "licence" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.

# Wiki page type handler
#	this file must be loaded into the Wiki namespace
#
# ----------------
# bibliography page type
# ----------------
#

proc bib-EditFormAttr {pageVar} {
	return ""
}

proc bib-EditForm {pageVar} {
	upvar $pageVar thePage
	GetSite $thePage(site) theSite
	
	cgi_puts [Dict theSite bibtexEntries]
	cgi_textarea source=$thePage(source) rows=30 cols=70
}

proc bib-SavePage {pageVar} {
	cgi_import source
	return $source
}

proc bib-PreviewPage {pageVar} {
	cgi_import source
	return $source
}

proc bib-Page2Html {cmd pageVar} {
	upvar $pageVar thePage
	set siteName $thePage(site)
	GetSite $siteName theSite
	GetFormat $siteName theFormat
	global Httpd

	# create bib file
	set source [GetPageContent thePage]
	set bibfile [file join $Httpd(tmp) wiki.bib]
	if [catch {open $bibfile w} fd] {
		cgi_puts "<b>Could not create file $bibfile to run BibTex: $fd</b>\n<hr>\n<pre>$source</pre>"
		return
	}
	puts -nonewline $fd $source
	close $fd

	# run script that calls bibtex
	# [mbl] 16july04 : added lookup for 'bibscript' in site config file, defaults to wikibib
	# added third parameter to call script: directory where wiki documents live
    	set bibscript wikibib
	if [info exists theSite(bibscript)] {
		set bibscript $theSite(bibscript)
	}

	set bibtex [file join $Httpd(bin) $bibscript]
	set bstdir [file join $Httpd(extlibrary) wiki types]
	if [catch {exec $bibtex $Httpd(tmp) $bstdir $theSite(root)} msg] {
		set msg [join [split $msg \n] "<br>\n"]
		cgi_puts "<b>Could not run BibTex: $msg</b>\n<hr>\n<pre>$source</pre>"
		return
	}

	# [mbl] 29july04: added check for wiki.html in tmp dir. 
	# If it exists, use it unprocessed, otherwise use wiki.bbl and post-process it
	set htmlfile [file join $Httpd(tmp) wiki.html]
	set postprocess 0
	if [catch {open $htmlfile r} fd] {
	    set postprocess 1
	}

	# get bbl file
	set bblfile [file join $Httpd(tmp) wiki.bbl]
	if [catch {open $bblfile r} fd] {
		cgi_puts "<b>Could not find BibTex output in $bblfile: $msg</b>\n<hr>\n<pre>$source</pre>"
		return
	}
	set text [read $fd]

	# a little bit of post-processing (probably a bit too rough...):
	#	turn ~ into non breaking space
	#	turn -- into -
	#	remove { and }
	#	remove trailing % and the \n after it, which are added by BibTex when splitting long lines
	if {$postprocess} {
		regsub -all {~} $text {\&nbsp;} text
		regsub -all -- {--} $text {-} text
		regsub -all {[{}]} $text {} text
		regsub -all {%
} $text {} text
		# note: Tcl wants a real newline in the search string (\n does not work).
	}

	# output JavaScript to support highlighting of selected reference
	# get hightlight markup and color from format
	set script {<script>
	HL=window.location.hash
	function HLon(str) {
	  if (str == HL) {
	    document.write("%on");
	  }
	}
	function HLoff(str) {
	  if (str == HL) {
	    document.write("%off");
	  }
	}
</script>}
	
	# %on and %off in above script are substituted by
	# proper markup from theFormat
	# The highlighOn markup itself is parameterized by the highlight
	# color, which is also defined in the format
	regsub -all "%color" $theFormat(highlightParOn) $theFormat(highlightColor) highlightParOn
	regsub -all "%on" $script $highlightParOn script
	regsub -all "%off" $script $theFormat(highlightParOff) script

if {$postprocess} {
	cgi_puts $script
}
	cgi_puts $text
	close $fd

	# clean up
	catch {file delete $bibfile}
	catch {file delete $htmlfile}
	catch {file delete $bblfile}
}

# turn a wiki link of the form [[bib:<path> <title>]] into HTML.
# return a triplet <url> <querystring> <anchor>
#
# for bib links:
#	[[bib:]] -> link to default bibliography page
#	[[bib: bibpage]] -> link to bibpage (same as [[bibpage]], for that matter).
#	[[bib:foo]] -> link to reference 'foo' in default bibliography page
#	[[bib:foo bibpage]] -> link to reference 'foo' in bibpage.
#
proc bib-Link {pageVar path title} {
	upvar $pageVar thePage
	GetSite $thePage(site) theSite

	# get default bib page if title not specified
	if {$title == ""} {
		if [catch {set $theSite(bibliography)} title] {
			set title [Dict theSite "bibliography"]
		}
	}

	set url [Title2Name $title].wiki
	set query ?[cgi_set cmd get]&[cgi_set anchor $title]&[cgi_set type bib]
	
	if {$path == ""} {
		# link to bib page
		set anchor $title
	} else {
		# link to entry in bib page
		set anchor "<span class=bibcite>\[$path\]</span>"
		# InternetExplorer JavaScript does not parse a URL of the form
		# path#hash?query properly. It needs to be path?query#hash
		# So we append to query instead of url so the script
		# in bib-Page2Html works on both Navigator and IE.
		append query "#$path"
	}

	return [list $url $query $anchor]
}

# -- Log--
#
# mbl 29/07/04	after bibtex is run, look for wiki.html. If exists, use it as is.
#		Otherwise, uses wiki.bbl and process it as before.
#
# mbl 16/07/04	added lookup for 'bibscript' in site config file, defaults to wikibib
#
# mbl 10/03/00	added <br> after message.
#
# mbl 24/01/00	moved dict entries to lang.tcl
#
# mbl 21/04/01	used entry "bibliography" in dictionary for default name
#
# mbl 22/04/01	added support to highlight the reference when the bibliography
#		page is displayed as a result of a citation link
#
# mbl 03/11/03	fixed bug in cgi_textarea for EditForm
#
