# Tcl Wiki Server - $Id: doc.tcl,v 1.8 2000/08/14 09:54:22 mbl Exp $
# 
# Copyright (c) 1997-99 Michel Beaudouin-Lafon, mbl@daimi.au.dk
# See the file "licence" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.

# Wiki page type handler
#	this file must be loaded into the Wiki namespace
#
# ----------------
# source documentation page type
# ----------------
#

proc ParseDocFile {source variables} {
	foreach var $variables {
		uplevel [list set $var {}]
	}
	foreach line [split $source \n] {
		if [regexp "^(.*)=(.*)$" $line dummy var val] {
			if {[lsearch $variables $var] >= 0} {
				uplevel [list set $var $val]
			}
		}
	}
}

proc doc-EditFormAttr {pageVar} {
	return ""
}

proc doc-EditForm {pageVar} {
	upvar $pageVar thePage
	
	ParseDocFile $thePage(source) {module files}

	# ### we should use the dictionary and format...
	h2 "Documentation page"
	p
	cgi_puts "Module: "
	cgi_text module=$module width=40
	p
	cgi_puts "Files: "
	cgi_textarea files=[join $files "\n"] rows=30 cols=70
}

proc doc-SavePage {pageVar} {
	cgi_import module
	cgi_import files
	return "module=$module\nfiles=[join [split $files \n] { }]"
}

proc doc-PreviewPage {pageVar} {
	cgi_import module
	cgi_import files
	return "module=$module\nfiles=[join [split $files \n] { }]"
}

proc doc-Page2Html {cmd pageVar} {
	upvar $pageVar thePage
	set siteName $thePage(site)
	GetSite $siteName theSite
	GetFormat $siteName theFormat

	ParseDocFile [GetPageContent thePage] {module files}

	# ### should use dictionary and Format
	h2 "Documentation of $module"
	bullet_list {
		li "<a href=\"$theSite(rootURL)/_doc/$module/inx.html\">Index</a>"
	}
	bullet_list {
		foreach file $files {
			set line [string trim $file " 	"]
			li "<a href=\"$theSite(rootURL)/_doc/$module/$file.html\">$file</a>"
		}
	}
	
	
	# now execute the doc generator in the background
	# this uses the Site(docpath) entry to be set in the site _config file.
	# this entry must be of the form <cvsroot> or {<cvsroot> <repository>}
	#	cvsroot is the CVSROOT directory
	#	repository, if specified, is the root module
	global Httpd
	set wikidoc [file join $Httpd(bin) wikidoc]
	set cvsroot ""
	set repository ""
	if [info exist theSite(docpath)] {
		# this will set rootmodule to "" if the list has only one element
		lset {cvsroot repository} $theSite(docpath)
	}
	set dest [file join $Httpd(root) $siteName _doc]
	file mkdir [file join $dest $module]
	set url [CurrentURL $siteName $thePage(name)]
	set outfile [file join $dest $module .wikidoc.out]

	cgi_puts $theFormat(hr)
	cgi_puts "<small><a href=\"$theSite(rootURL)/_doc/$module/.wikidoc.out\">Output from wikidoc</a> (in case of error).</small>"
	
	global DebugLevel
	# ### add -debug only if debug is on
	catch {eval exec $wikidoc -debug -wikiroot $theSite(rootURL) -cvsroot $cvsroot -repository $repository -module $module -dest $dest -tocurl $url $files >& $outfile &}
}

# -- Log --
#
# mbl 19/10/99	added -wikiroot argument to wikidoc
#
# mbl 07/07/99	added "repository" variable,
#		captured output from wikidoc into a file and added a link to this file in the page
#
# mbl 24/01/00	moved dict entries to lang.tcl
#
