# Tcl Wiki Server - $Id: sitemap.tcl,v 1.4 1999/07/07 15:20:32 mbl Exp $
# 
# Copyright (c) 1997-99 Michel Beaudouin-Lafon, mbl@daimi.au.dk
# See the file "licence" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.

# Wiki page type handler
#	this file must be loaded into the Wiki namespace
#
# ----------------
# sitemap format - used to edit the list of categories to display in the sitemap
# ----------------
#
proc sitemap-EditFormAttr {pageVar} {
	return ""
}

proc sitemap-EditForm {pageVar} {
	upvar $pageVar thePage
	set site $thePage(site)
	GetSite $site theSite
	
	set source $thePage(source)
	
	# add categories that are not there yet
	# note that normally the sitemap is edited after having
	# been displayed, which ensures that the list of categories
	# is up to date.
	set categories {}
	foreach line [split $source \n] {
		set cat [string trim $line " 	*=-_"]
		if {$cat != ""} {
			lappend categories $cat
		}
	}
	foreach cat $theSite(categories) {
		if {$cat != "" && [lsearch  -exact $categories $cat] < 0} {
			lappend newcat $cat
		}
	}
	if [info exist newcat] {
		if {$source != ""} {
			append source "---\n"
		}
		append source "[join $newcat \n]"
	}
	
	# textarea to edit list of categories
	cgi_textarea source=$source rows=30 cols=70
}

proc sitemap-SavePage {pageVar} {
	upvar $pageVar thePage
	GetSite $thePage(site) theSite

	set thePage(title) [Dict theSite siteMap $thePage(site)]
	cgi_import source
	return $source
}

proc sitemap-PreviewPage {pageVar} {
	upvar $pageVar thePage
	GetSite $thePage(site) theSite

	set thePage(title) [Dict theSite siteMap $thePage(site)]
	cgi_import source
	return $source
}

proc sitemap-Page2Html {cmd pageVar} {
	upvar $pageVar thePage
	set site $thePage(site)
	GetSite $site theSite
	GetFormat $site theFormat

	# create list of pages by category
	# ### the effect of this is to load _all_ pages
	# ### maybe we could unload the pages that were not loaded before
	# ### or have a general mechanism to unload unused pages, based on last access date
	foreach p [AllPages theSite] {
		if {$p == "_sitemap"} {
			# we don't want the site map in the site map...
			continue
		}
		GetPage $site $p aPage
		lappend category($aPage(category)) [list $p [PageRef aPage]]
		# check if the page has the same title as the category
		if {[Title2Name $aPage(category)] == $p} {
			set title($aPage(category)) [PageLink aPage]
		}
	}
# puts "sitemap"
# parray category
	
	# create list of categories
	set categories {}
	set formattedlist {}
	foreach cat [split [GetPageContent thePage] \n] {
		lappend formattedlist $cat
		set cat [string trim $cat " 	*-=_"]
		if {$cat != ""} {
			lappend categories $cat
		}
	}
#puts "categories=$categories"
	
	foreach cat [lsort [array names category]] {
		if {$cat != "" && [lsearch -exact $categories $cat] < 0} {
			lappend newcat $cat
		}
	}
	if [info exist newcat] {
#puts "new categories=$newcat"
		lappend formattedlist {}
		eval lappend formattedlist $newcat
		eval lappend categories $newcat
	}
	
	# update the list of categories since we have it at hand
	set theSite(categories) $categories
	
	# create sitemap
	lset {open item close} $theFormat(pageList)
	
	# - emtpy category first
#puts "sitemap"
	set cat {}
	if [info exist category($cat)] {
#puts "  pages with no category"
		cgi_puts $open
		foreach pr [lsort $category($cat)] {
			lset {p ref} $pr
# puts "    - $ref"
			cgi_puts $item$ref
		}
		cgi_puts $close
	}
	
	# - then categories as they appear in the formatted list
#puts "formattedlist=$formattedlist"
	foreach cat $formattedlist {
#puts "  category $cat"
		# get indentation 
		# ### ignored for now ###
		regsub {(^[ 	]*).*$} $cat {\1} indent
		
		# handle empty and separator lines
		if [regexp {^[ 	]*$} $cat] {		
			cgi_br
			continue
		}
		if [regexp {^-*$} $cat] {		
			cgi_puts $theFormat(hr)
			continue
		}
		
		# extract category and heading level
		if [regsub {^\*\*+ +(.*) +\*\*+$} $cat {\1} cat] {
			set format h1
		} elseif [regsub {^==+ +(.*) +==+$} $cat {\1} cat] {
			set format h2
		} elseif [regsub {^--+ +(.*) +--+$} $cat {\1} cat] {
			set format h3
		} elseif [regsub {^___+ +(.*) +___+$} $cat {\1} cat] {
			set format h4
		} else {
			set format categoryHeader
		}
#puts "    format is $format"
		
		# header for the category
		# make it a link if there is a page in this category with the same name
		if [info exist title($cat)] {
			regsub {(.*)} $title($cat) $theFormat($format) line
		} else {
			regsub {(.*)} $cat $theFormat($format) line
		}
		cgi_puts $line
#puts "    line is $line"
		
		# list of pages
		if [info exist category($cat)] {
			cgi_puts $open
			foreach pr [lsort $category($cat)] {
				lset {p ref} $pr
# puts "    - $ref"
				cgi_puts $item$ref
			}
			cgi_puts $close
		}
	}
#puts "done sitemap"
}

###
### il reste a changer le menu des categories, et a traiter plusieurs niveaux de headers
### dans la sitemap (peut-etre en traduisant la site map en wiki puis en appelant Page2Html...)
### pour le menu, on peut peut-etre utiliser Wiki(categories) ou alors ajouter Wiki(catmenu)
### 
### voir ce qui se passe etant donne que c'est maintenant un type: lien [[sitemap:foobar]]
### ca donne quoi ? Doit permettre de creer plusieurs site maps, voire de lier vers des
### sitemaps d'autres sites: [[sitemap:]] sitemap par defaut de ce site
### [[sitemap:wiki]] sitemap par defaut du site wiki, [[sitemap: liste des images]] la
### site definie dans la page liste_des_images.wiki. Il faut alors ajouter les pages non
### referencees seulement dans la site map par defaut.
###
### creer la page a la creation du site
### voir si on veut la mettre dans la sitemap elle-meme (why not?)
### est-ce un probleme si son nom ne correspond pas a son titre (e.g. Carte du site vs sitemap)
###
### ajouter au debut de la site map un formulaire de recherche.
###

# -- Log --
#
# mbl 29/06/99	commented out puts
#
# mbl 30/06/99	added a test to not list the sitemap page in the site map.
#
# mbl 07/07/99	minor change to the display of categories in the edit form
#
