/*
 * (c) 2000, Michel Beaudouin-Lafon, mbl@daimi.au.dk
 *
 * This program is used to launch the wiki server so that
 * it accepts connections on port 80 (like a normal Web server)
 * without the server itselef running as a privileged user.
 * 
 * Since port 80 is privileged, it can only be opened by a
 * privileged user, but we don't want the whole wiki server
 * to run as a privileged user. So this program creates the
 * socket, binds it to port 80, then exec's the wiki server
 * with normal privileges.
 *
 * This requires a patch to tclsh so that a server socket can
 * be created from an existing file descriptor. The patch is
 * in tcl8.0.4/unix/tclUnixChan.c, in function CreateSocket.
 * If the host name is a single number, i.e. "3", instead of
 * an internet host name or IP number, it is taken as the
 * file descriptor of an open socket.
 * 
 */

#include "stdio.h"
#include <sys/socket.h>		/* struct sockaddr, SOCK_STREAM, ... */
#include <netinet/in.h>		/* struct in_addr, struct sockaddr_in */
#include <netdb.h>			/* gethostbyname() */

char* argv0;

void usage () 
{
	fprintf (stderr, "usage: %s [host [port [prog]]]\n", argv0);
	exit (1);
}

/* from tcl8.0.4/unix/tclUnixChan.c */
CreateSocketAddress(sockaddrPtr, host, port)
    struct sockaddr_in *sockaddrPtr;	/* Socket address */
    char *host;				/* Host.  NULL implies INADDR_ANY */
    int port;				/* Port number */
{
    struct hostent *hostent;		/* Host database entry */
    struct in_addr addr;		/* For 64/32 bit madness */

    (void) memset((void *) sockaddrPtr, '\0', sizeof(struct sockaddr_in));
    sockaddrPtr->sin_family = AF_INET;
    sockaddrPtr->sin_port = htons((unsigned short) (port & 0xFFFF));
    if (host == NULL) {
	addr.s_addr = INADDR_ANY;
    } else {
        addr.s_addr = inet_addr(host);
        if (addr.s_addr == -1) {
            hostent = gethostbyname(host);
            if (hostent != NULL) {
                memcpy((void *) &addr,
                        (void *) hostent->h_addr_list[0],
                        (size_t) hostent->h_length);
            } else {
#ifdef	EHOSTUNREACH
                errno = EHOSTUNREACH;
#else
#ifdef ENXIO
                errno = ENXIO;
#endif
#endif
                return 0;	/* error */
            }
        }
    }
        
    /*
     * NOTE: On 64 bit machines the assignment below is rumored to not
     * do the right thing. Please report errors related to this if you
     * observe incorrect behavior on 64 bit machines such as DEC Alphas.
     * Should we modify this code to do an explicit memcpy?
     */

    sockaddrPtr->sin_addr.s_addr = addr.s_addr;
    return 1;	/* Success. */
}

int main (int argc, char** argv) 
{
	/* default values for program arguments */
	char* host = "wiki.daimi.au.dk";
	int port = 80;
	char* prog = "/users/wiki/bin/wiki";

	/* local variables */
    struct sockaddr_in sockaddr;
	int sock;
	char* eos;
	char sockString[10];
	char portString[10];
	
	argv0 = argv[0];
	
	/* parse arguments */
	if (argc > 4)
		usage ();

	if (argc >= 2)
		host = argv[1];

	if (argc >= 3) {
		port = strtol (argv[2], &eos, 10);
		if (eos != argv[2] + strlen (argv[2])) {
			fprintf (stderr, "bad port number: %s\n", argv[2]);
			usage ();
		}
	}
	
	if (argc >= 4)
		prog = argv[3];
	
	/* create address */
	if (! CreateSocketAddress (&sockaddr, host, port)) {
		perror ("could not create address");
		exit (1);
	}
	
	/* create socket */
	sock = socket (AF_INET, SOCK_STREAM, 0);
	if (sock < 0) {
		perror ("could not open socket");
		exit (1);
	}
	
	/* bind */
	if (bind (sock, (struct sockaddr *) &sockaddr, sizeof(struct sockaddr)) < 0) {
		perror ("could not bind socket");
		exit (1);
	}

	/* *** DO WE NEED TO DO ANYTHING ABOUT PRIVILEGES, i.e. setuid??? *** */
	/* exec script */
	sprintf (sockString, "%d", sock);
	sprintf (portString, "%d", port);
	execl (prog, prog, "-host", host, "-port", portString, "-serversock", sockString, (char*) NULL);

	/* in case exec fails */
	perror (prog);
	exit (1);
}
